 ##########################################################################
 # 
 # Get-VMwareVMList
 #
 ##########################################################################

 <#
.SYNOPSIS
Retrieves physical host and virtual machine data from a VMware vSphere or vCenter server

.DESCRIPTION
The Get-VMwareVMList script queries a single vSphere or vCenter server and produces a CSV file
including virtual machine and physical host details. The file (VMwareData.csv) contains one 
record per virtual machine. Data collected includes
	VM Name
	VM CPU, Memory & Network details
	Physical Host Name
	Physical Host CPU, Memory, Network, Cluster & vMotion details

If a vCenter server is queried, details for all VMs in the farm are retrieved.

.PARAMETER Server 
Host name of vSphere or vCenter server to scan
.PARAMETER Username
VMware Username (Required)
e.g. root (for vSphere server i.e. local account)
     jon.mulligan (for vCenter server i.e. Windows domain account)

.PARAMETER Password
VMware Password (Required)

.EXAMPLE
Get all guest & host info from from the farm managed by the vSphere server 'Reliant'. 
Get-VMwareVMList VMserver Reliant

#>

Param(
	[alias("o1")]
    $OutputFile1 = "VMwareVMs.csv",
	[alias("o2")]
    $OutputFile2 = "VMwareHosts.csv",
	[alias("o3")]
    $OutputFile3 = "VMwareClusters.csv",
	[alias("o4")]
    $OutputFile4 = "VMwareVMDRSRules.csv",
	[alias("o5")]
    $OutputFile5 = "VMwareVMDRSGroups.csv",
	[alias("o6")]
    $OutputFile6 = "VMwareVMotionData.csv",
	[alias("username")]
	$VMwareUsername,
	[alias("password")]
	$VMwarePassword,
	[alias("server")]
	$VMserver,
	[ValidateSet("VMData","VMDataPlusDRS","VMMigrationHistory","AllData")]
	$RequiredData = "VMDataPlusDRS",
	[alias("VMotionHistoryDaysValue")]
	$VMotionHistoryDays = 14,
    $VMServerPort = -1,
    $VMServerProtocol = "",
    $SkipProxy = $true,
	[alias("log")]
	[string]$LogFile = "VMLogFile.txt",
	[switch]$Verbose)

function LogText {
	param(
		[Parameter(Position=0, ValueFromRemainingArguments=$true, ValueFromPipeline=$true)]
		[Object] $Object,
		[System.ConsoleColor]$color = [System.Console]::ForegroundColor,
		[switch]$NoNewLine = $false 
	)

	# Display text on screen
	Write-Host -Object $Object -ForegroundColor $color -NoNewline:$NoNewLine

	if ($LogFile) {
		$Object | Out-File $LogFile -Encoding utf8 -Append 
	}
}

function InitialiseLogFile {
	if ($LogFile -and (Test-Path $LogFile)) {
		Remove-Item $LogFile
	}
}

function LogProgress($progressDescription){
	if ($Verbose){
		LogText ""
	}

	$output = Get-Date -Format HH:mm:ss.ff
	$output += " - "
	$output += $progressDescription
	LogText $output -Color Green
}

function LogError([string[]]$errorDescription){
	if ($Verbose){
		LogText ""
	}

	$output = Get-Date -Format HH:mm:ss.ff
	$output += " - "
	$output += $errorDescription -join "`r`n              "
	LogText $output -Color Red
	Start-Sleep -s 3
}

function LogLastException() {
    $currentException = $Error[0].Exception;

    while ($currentException)
    {
        LogText -Color Red $currentException
        LogText -Color Red $currentException.Data
        LogText -Color Red $currentException.HelpLink
        LogText -Color Red $currentException.HResult
        LogText -Color Red $currentException.Message
        LogText -Color Red $currentException.Source
        LogText -Color Red $currentException.StackTrace
        LogText -Color Red $currentException.TargetSite

        $currentException = $currentException.InnerException
    }

	Start-Sleep -s 3
}

function QueryUser([string]$Message, [string]$Prompt, [switch]$AsSecureString = $false, [string]$DefaultValue){
	if ($Message) {
		LogText $Message -color Yellow
	}

	if ($DefaultValue) {
		$Prompt += " (Default [$DefaultValue])"
	}

	$Prompt += ": "
	LogText $Prompt -color Yellow -NoNewLine
	$strResult = Read-Host -AsSecureString:$AsSecureString

	if(!$strResult -or ($strResult.Length -eq 0)) {
		$strResult = $DefaultValue
	}

	return $strResult
}

function Get-ConsoleCredential([String] $Message, [String] $DefaultUsername)
{
	$strUsername = QueryUser -Message $Message -Prompt "Username" -DefaultValue $DefaultUsername
	if (!$strUsername){
		return $null
	}

	$strSecurePassword = QueryUser -Prompt "Password" -AsSecureString
	if (!$strSecurePassword){
		return $null
	}

	return new-object Management.Automation.PSCredential $strUsername, $strSecurePassword
}
                                                                          
function LogEnvironmentDetails {
	LogText -Color Gray " "
	LogText -Color Gray "  ###########################################################"
	LogText -Color Gray "  #       _______     __                                    #"
	LogText -Color Gray "  #      |  ___\ \   / /                                    #"
	LogText -Color Gray "  #      | |__  \ \_/ /                                     #"
	LogText -Color Gray "  #      |  __|  \   /                  Ernst & Young       #"
	LogText -Color Gray "  #      | |____  | |        Windows Inventory Script       #"
	LogText -Color Gray "  #      |______| |_|                                       #"
	LogText -Color Gray "  #                                                         #"
	LogText -Color Gray "  ###########################################################"
	LogText -Color Gray " "
	LogText -Color Gray " Get-VMwareVMList.ps1"
	LogText -Color Gray " "

	$OSDetails = Get-WmiObject Win32_OperatingSystem
	LogText -Color Gray "Computer Name:        $($env:COMPUTERNAME)"
	LogText -Color Gray "User Name:            $($env:USERNAME)@$($env:USERDNSDOMAIN)"
	LogText -Color Gray "Windows Version:      $($OSDetails.Caption)($($OSDetails.Version))"
	LogText -Color Gray "PowerShell Host:      $($host.Version.Major)"
	LogText -Color Gray "PowerShell Version:   $($PSVersionTable.PSVersion)"
	LogText -Color Gray "PowerShell Word size: $($([IntPtr]::size) * 8) bit"
	LogText -Color Gray "CLR Version:          $($PSVersionTable.CLRVersion)"
	LogText -Color Gray "Current Date Time:    $(Get-Date -Format "yyyy-MM-dd HH:mm:ss")"
	LogText -Color Gray "Username Parameter:   $VMwareUsername"
	LogText -Color Gray "Server Parameter:     $VMserver"
	LogText -Color Gray "Required Data:        $RequiredData"
	LogText -Color Gray "Output File 1:        $OutputFile1"
	LogText -Color Gray "Output File 2:        $OutputFile2"
	LogText -Color Gray "Output File 3:        $OutputFile3"
	LogText -Color Gray "Output File 4:        $OutputFile4"
	LogText -Color Gray "Output File 5:        $OutputFile5"
	LogText -Color Gray "Output File 6:        $OutputFile6"
	LogText -Color Gray "Log File:             $LogFile"
	LogText -Color Gray ""
}

function SetupDateFormats {
    # Standardise date/time output to ISO 8601'ish format
    $bDateFormatConfigured = $false
    $currentThread = [System.Threading.Thread]::CurrentThread
    
    try {
        $CurrentThread.CurrentCulture.DateTimeFormat.ShortDatePattern = 'yyyy-MM-dd'
        $CurrentThread.CurrentCulture.DateTimeFormat.LongDatePattern = 'yyyy-MM-dd HH:mm:ss'
        $bDateFormatConfigured = $true
    }
    catch {
    }

    if (!($bDateFormatConfigured)) {
        try {
            $cultureCopy = $CurrentThread.CurrentCulture.Clone()
            $cultureCopy.DateTimeFormat.ShortDatePattern = 'yyyy-MM-dd'
            $cultureCopy.DateTimeFormat.LongDatePattern = 'yyyy-MM-dd HH:mm:ss'
            $currentThread.CurrentCulture = $cultureCopy
        }
        catch {
        }
    }
}      

function Get-VMwareVMList
{
	try
	{
        SetupDateFormats
		InitialiseLogFile
		LogEnvironmentDetails

		if (!(EnvironmentConfigured))
		{
            LogProgress "Importing modules/snap-ins"
			$allSnapIns = get-pssnapin -registered | sort -Descending
			foreach ($snapIn in $allSnapIns){
				if ($snapIn.name -eq 'VMware.VimAutomation.Core') {
					LogText "Adding SnapIn: $($snapIn.Name)"
					add-PSSnapin -Name $snapIn.name
				}
			}
		}

        if (!(EnvironmentConfigured))
		{
            if (Get-Module -ListAvailable -Name VMware.VimAutomation.Core) {
                LogText "Adding Module: VMware.VimAutomation.Core"
                Import-Module -Name VMware.VimAutomation.Core
            }
        }

		if (!(EnvironmentConfigured))
		{
			LogError("VMware PowerShell modules could not be loaded", 
				"Please ensure that VMware PowerCLI module is installed on this computer",
			    "https://www.vmware.com/support/developer/PowerCLI/",
                "https://my.vmware.com/group/vmware/details?downloadGroup=PCLI650R1&productId=614")
			
			return
		}

		# Configure the environment
		try
		{
			LogProgress "Configuring Connection Settings" 
            if ($SkipProxy) {
                Set-PowerCLIConfiguration -ProxyPolicy NoProxy -Confirm:$false -ErrorVariable $errorConfig
            }
			Set-PowerCLIConfiguration -InvalidCertificateAction ignore -Confirm:$false -ErrorVariable $errorConfig
		}
		catch
		{
			LogError("Unable to configure connection settings. Continuing")
		}

		if (!($VMserver)){
			$VMserver = QueryUser -Message "VMware Server Name Required" -Prompt "VMware host or vCenter Name" -DefaultValue $env:computerName
		}

		if(!($VMwareUsername -and $VMwarePassword)){
			$Creds = Get-ConsoleCredential -Message "VMware Credentials Required" -DefaultUsername $VMwareUsername
			if (!($Creds)) {
				LogError("Missing Parameter: Username and Password must be specified")
				return
			}
		}
	}
	catch 
	{
		LogLastException
        return
	}
    
    try
    {
		# Connect to the VMware server
        LogProgress "Connecting to VMware server"
        $parameters = @{
            Server = $VMServer
        }
        if ($VMServerPort -ne -1) {
            $parameters.Add("Port", $VMServerPort);
        }
        if ($VMServerProtocol -ne "") {
            $parameters.Add("Protocol", $VMServerProtocol);
        }
		if ($Creds) {
            $parameters.Add("Credential", $Creds);
		}
		else {
            $parameters.Add("User", $VMwareUsername);
            $parameters.Add("Password", $VMwarePassword);
		}

        $viServer = Connect-VIServer @parameters
        
		if($viServer){
			LogProgress "Connected to VMware server"
		}
		else {
			Throw "VMware logon failed"
		}
    }
    catch
    {
        LogLastException
        return
    }
	
    # Get the VM & VM Host Data
    try
    {
		if ($RequiredData -eq "VMData" -or $RequiredData -eq "VMDataPlusDRS" -or $RequiredData -eq "AllData") {
			LogProgress "Getting VM Info"

			$DataCenters = Get-Datacenter | Sort-Object -Property Name

			$VmInfo = ForEach ($Datacenter in ($DataCenters)) {
				ForEach ($VM in ($Datacenter | Get-VM | Sort-Object -Property Name)) {
                
					$IPAddresses = "";
					$MACAddresses = "";
					$NetworkNames = "";
					foreach ($NIC in $VM.Guest.Nics) {
						$IPAddresses += $NIC.IPAddress -join ','
						$MACAddresses += $NIC.MacAddress
						$NetworkNames += $NIC.NetworkName
                
						$IPAddresses += ','
						$MACAddresses += ','
						$NetworkNames += ','
					}
                
					"" | Select-Object -Property @{N="VM";E={$VM.Name}},
					@{N="VMCPUCount";E={$vm.ExtensionData.Config.Hardware.NumCPU/$vm.ExtensionData.Config.Hardware.NumCoresPerSocket}},
					@{N="VMCPUCoreCount";E={$vm.NumCPU}},
					@{N="VMCPUCorePerCPUCount";E={$vm.ExtensionData.Config.Hardware.NumCoresPerSocket}},
					@{N="PowerState";E={$VM.PowerState}},
					@{N="FaultToleranceState";E={$VM.ExtensionData.Summary.Runtime.FaultToleranceState}},
					@{N="OnlineStandby";E={$VM.ExtensionData.Summary.Runtime.OnlineStandby}},
					@{N="Version";E={$VM.Version}},
					@{N="Description";E={$VM.Description}},
					@{N="Notes";E={$VM.Notes}},
					@{N="MemoryMB";E={$VM.MemoryMB}},
					@{N="ResourcePool";E={$VM.ResourcePool}},
					@{N="ResourcePoolID";E={$VM.ResourcePoolId}},
					@{N="PersistentID";E={$VM.PersistentId}},
					@{N="ID";E={$VM.Id}},
					@{N="UID";E={$VM.Uid}},
					@{N="UUID";E={$VM.ExtensionData.Config.Uuid}},
					@{N="IPs";E={$IPAddresses}},
					@{N="MACs";E={$MACAddresses}},
					@{N="NetworkNames";E={$NetworkNames}},
					@{N="OS";E={$VM.Guest.OSFullName}},
					@{N="FQDN";E={$VM.Guest.HostName}},
					@{N="ScreenDimensions";E={$VM.Guest.ScreenDimensions}},
					@{N="OS2";E={$VM.ExtensionData.Guest.OSFullName}},
					@{N="FQDN2";E={$VM.ExtensionData.Guest.HostName}},
					@{N="GuestHostname";E={$VM.ExtensionData.Summary.Guest.HostName}},
					@{N="GuestId";E={$VM.ExtensionData.Summary.Guest.GuestId}},
					@{N="GuestFullName";E={$VM.ExtensionData.Summary.Guest.GuestFullName}},
					@{N="GuestIP";E={$VM.ExtensionData.Summary.Guest.IpAddress}},
					@{N="Datacenter";E={$Datacenter.name}},
					@{N="Cluster";E={$vm.VMHost.Parent.Name}},
					@{N="ClusterID";E={$vm.VMHost.Parent.ID}},
					@{N="HostName1";E={$vm.VMHost.Name}},
					@{N="HostName2";E={$vm.VMHost.ExtensionData.Summary.Config.Name}},
					@{N="HostName3";E={$VM.VMHost.NetworkInfo.HostName}},
					@{N="HostDomainName";E={$VM.VMHost.NetworkInfo.DomainName}},
					@{N="HostCPUCount";E={$vm.VMHost.ExtensionData.Summary.Hardware.NumCpuPkgs}},
					@{N="HostCPUCoreCount";E={$vm.VMHost.ExtensionData.Summary.Hardware.NumCpuCores/$vm.VMHost.ExtensionData.Summary.Hardware.NumCpuPkgs}},
					@{N="HyperthreadingActive";E={$vm.VMHost.HyperthreadingActive}},
					@{N="HostManufacturer";E={$VM.VMHost.Manufacturer}},
					@{N="HostID";E={$VM.VMHost.Id}},
					@{N="HostUID";E={$VM.VMHost.Uid}},
					@{N="HostPowerState";E={$VM.VMHost.PowerState}},
					@{N="HostHyperThreading";E={$VM.VMHost.HyperthreadingActive}},
					@{N="HostvMotionEnabled";E={$vm.VMHost.ExtensionData.Summary.Config.VmotionEnabled}},
					@{N="HostVendor";E={$VM.VMHost.ExtensionData.Hardware.SystemInfo.Vendor}},
					@{N="HostModel";E={$VM.VMHost.ExtensionData.Hardware.SystemInfo.Model}},
					@{N="HostRAM";E={$VM.VMHost.ExtensionData.Summary.Hardware.MemorySize}},
					@{N="HostCPUModel";E={$VM.VMHost.ExtensionData.Summary.Hardware.CpuModel}},
					@{N="HostThreadCount";E={$VM.VMHost.ExtensionData.Summary.Hardware.NumCpuThreads}},
					@{N="HostCPUSpeed";E={$VM.VMHost.ExtensionData.Summary.Hardware.CpuMhz}},
					@{N="HostProductName";E={$VM.VMHost.ExtensionData.Summary.Config.Product.Name}},
					@{N="HostProductVersion";E={$VM.VMHost.ExtensionData.Summary.Config.Product.Version}},
					@{N="HostProductBuild";E={$VM.VMHost.ExtensionData.Summary.Config.Product.Build}},
					@{N="HostProductOS";E={$VM.VMHost.ExtensionData.Summary.Config.Product.OsType}},
					@{N="HostProductLicenseKey";E={$VM.VMHost.LicenseKey}},
					@{N="HostProductLicenseName";E={$VM.VMHost.ExtensionData.Summary.Config.Product.LicenseProductName}},
					@{N="HostProductLicense Version";E={$VM.VMHost.ExtensionData.Summary.Config.Product.LicenseProductVersion}},
					@{N="HostvMotionIPAddress";E={$VM.VMHost.ExtensionData.Config.vMotion.IPConfig.IpAddress}},
					@{N="HostvMotionSubnetMask";E={$VM.VMHost.ExtensionData.Config.vMotion.IPConfig.SubnetMask}}   
				}
			}
			$VmInfo | Export-Csv -NoTypeInformation -Path $OutputFile1 -Encoding UTF8

			$VmHostInfo = ForEach ($Datacenter in ($DataCenters)) {
				ForEach ($VMHost in ($Datacenter | Get-VMHost | Sort-Object -Property Name)) {
                
                    $strUuid = ""
                    try {
                        $EsxCli = Get-EsxCli -VMHost $VMHost
		                $Uuid = $EsxCli.system.uuid.get()
                        $strUuid = $Uuid.ToString();
                    }
                    catch {
                    }
                    
					"" | Select-Object -Property @{N="VMHost";E={$VMHost.Name}},
					@{N="Name2";E={$VMHost.ExtensionData.Summary.Config.Name}},
					@{N="Name3";E={$VMHost.NetworkInfo.HostName}},
					@{N="ID";E={$VMHost.Id}},
					@{N="UID";E={$VMHost.Uid}},
                    @{N="HardwareUUID";E={$VMHost.ExtensionData.Hardware.SystemInfo.Uuid}},
                    @{N="VMwareUUID";E={$strUuid}},
					@{N="Datacenter";E={$Datacenter.name}},
					@{N="Cluster";E={$VMHost.Parent.Name}},
					@{N="ClusterID";E={$VMHost.Parent.ID}},
                    @{N="IsStandalone";E={$VMHost.IsStandalone}},
					@{N="DomainName";E={$VMHost.NetworkInfo.DomainName}},
					@{N="CPUCount";E={$VMHost.ExtensionData.Summary.Hardware.NumCpuPkgs}},
					@{N="CPUCoreCount";E={$VMHost.ExtensionData.Summary.Hardware.NumCpuCores/$VMHost.ExtensionData.Summary.Hardware.NumCpuPkgs}},
					@{N="HyperthreadingActive";E={$VMHost.HyperthreadingActive}},
					@{N="Manufacturer";E={$VMHost.Manufacturer}},
					@{N="PowerState";E={$VMHost.PowerState}},
					@{N="vMotionEnabled";E={$VMHost.ExtensionData.Summary.Config.VmotionEnabled}},
					@{N="Vendor";E={$VMHost.ExtensionData.Hardware.SystemInfo.Vendor}},
					@{N="Model";E={$VMHost.ExtensionData.Hardware.SystemInfo.Model}},
					@{N="RAM";E={$VMHost.ExtensionData.Summary.Hardware.MemorySize}},
					@{N="CPUModel";E={$VMHost.ExtensionData.Summary.Hardware.CpuModel}},
					@{N="ThreadCount";E={$VMHost.ExtensionData.Summary.Hardware.NumCpuThreads}},
					@{N="CPUSpeed";E={$VMHost.ExtensionData.Summary.Hardware.CpuMhz}},
					@{N="ProductName";E={$VMHost.ExtensionData.Summary.Config.Product.Name}},
					@{N="ProductVersion";E={$VMHost.ExtensionData.Summary.Config.Product.Version}},
					@{N="ProductBuild";E={$VMHost.ExtensionData.Summary.Config.Product.Build}},
					@{N="ProductOS";E={$VMHost.ExtensionData.Summary.Config.Product.OsType}},
                    @{N="InstallDate";E={CalculateInstallDate($strUuid)}},
					@{N="ProductLicenseKey";E={$VMHost.LicenseKey}},
					@{N="ProductLicenseName";E={$VMHost.ExtensionData.Summary.Config.Product.LicenseProductName}},
					@{N="ProductLicense Version";E={$VMHost.ExtensionData.Summary.Config.Product.LicenseProductVersion}},
					@{N="vMotionIPAddress";E={$VMHost.ExtensionData.Config.vMotion.IPConfig.IpAddress}},
					@{N="vMotionSubnetMask";E={$VMHost.ExtensionData.Config.vMotion.IPConfig.SubnetMask}}   
				}
			}
			$VmHostInfo | Export-Csv -NoTypeInformation -Path $OutputFile2 -Encoding UTF8
		}
	}
    catch
    {
        LogLastException
    }

	try {
		if ($RequiredData -eq "VMDataPlusDRS" -or $RequiredData -eq "AllData") {

			LogProgress "Getting VM DRS Info"

			$AllClusters = Get-Cluster
			if ($AllClusters){
				LogProgress "$($AllClusters.Count) Cluster(s) found"

				# Save Cluster Details
				$ClusterInfo = $AllClusters | select Name, ID, UID, ParentID, HAEnabled, DrsEnabled, DrsMode, DrsAutomationLevel
				$ClusterInfo | Export-Csv -NoTypeInformation -Path $OutputFile3 -Encoding UTF8 

				$DRSRules = Get-DrsRule -Cluster $AllClusters

				if ($DRSRules){
					
					$ExtendedDRSRules = ForEach ($DRSRule in $DRSRules) {
                
						$VMIDs = $DRSRule.VMIds -join ','
						$VMIDs2 = ""
						$VMUIDs = ""
						$VMUUIDs = ""
						$AffineHostGroupName = ""
						$AntiAffineHostGroupName = ""
						$VmGroupName = ""

						if ($DRSRule.Type -eq "VMAffinity" -or $DRSRule.Type -eq "AntiVMAffinity") {
							$VMIDs2 = ($DRSRule.ExtensionData.VM | select Id) -join ','
							$VMUIDs = ($DRSRule.ExtensionData.VM | select Uid) -join ','
							#$VMUUIDs = ($DRSRule.ExtensionData.VM | select @{Name="UUID"; Expression={$_.ExtensionData.Config.Uuid}} | select -ExpandProperty UUID) -join ','
							#$VMUUIDs = ($DRSRule.ExtensionData.VM | select @{Name="UUID"; Expression={$_.ExtensionData.Config.Uuid}} | select -ExpandProperty UUID) -join ','
						}
						elseif ($DRSRule.Type -eq "VMHostAffinity") {
							$AffineHostGroupName = $DRSRule.ExtensionData.affineHostGroupName
							$AntiAffineHostGroupName = $DRSRule.ExtensionData.antiAffineHostGroupName
							$VmGroupName = $DRSRule.ExtensionData.vmGroupName
						}

						"" | Select-Object -Property @{N="Name";E={$DRSRule.Name}},
							@{N="Key";E={$DRSRule.Key}},
							@{N="Uid";E={$DRSRule.Uid}},
							@{N="Cluster";E={$DRSRule.Cluster}},
							@{N="ClusterId";E={$DRSRule.ClusterId}},
							@{N="ClusterUid";E={$DRSRule.ClusterUid}},
							@{N="Enabled";E={$DRSRule.Enabled}},
							@{N="Mandatory";E={$DRSRule.Mandatory}},
							@{N="Type";E={$DRSRule.Type}},
							@{N="VMIDs";E={$VMIDs}},
							@{N="VMIDs2";E={$VMIDs2}},
							@{N="VMUIDs";E={$VMUIDs}},
							@{N="VMUUIDs";E={$VMUUIDs}},
							@{N="AffineHostGroupName";E={$AffineHostGroupName}},
							@{N="AntiAffineHostGroupName";E={$AntiAffineHostGroupName}},
							@{N="VmGroupName";E={$VmGroupName}}
					}

					$ExtendedDRSRules | Export-Csv -NoTypeInformation -Path $OutputFile4 -Encoding UTF8
				}

				$ExpandedDRSGroups = @()
				ForEach ($VMwareCluster in $AllClusters) {

					$DRSGroups = $VMwareCluster.ExtensionData.ConfigurationEx.Group

					if ($DRSGroups){
					
						$ExpandedDRSGroups += ForEach ($DRSGroup in $DRSGroups) {
                
							$VMIDs = ""
							$VMUIDs = ""
							$VMNames = ""
							$HostIDs = ""
							$HostUIDs = ""
							$HostNames = ""

							if ($DRSGroup.VM) {
								$VMIDs = ($DRSGroup.VM | select Id) -join ','
								$VMUIDs = ($DRSGroup.VM | select Uid) -join ','
								$VMNames = ($DRSGroup.VM | select Name) -join ','
							}

							if ($DRSGroup.Host) {
								$HostIDs = ($DRSGroup.Host | select Id) -join ','
								$HostUIDs = ($DRSGroup.Host | select Uid) -join ','
								$HostNames = ($DRSGroup.Host | select Name) -join ','
							}

							"" | Select-Object -Property @{N="Name";E={$DRSGroup.Name}},
								@{N="Type";E={$DRSGroup.GetType()}},
								@{N="ClusterName";E={$VMwareCluster.Name}},
								@{N="ClusterID";E={$VMwareCluster.Id}},
								@{N="VMIDs";E={$VMIDs}},
								@{N="VMUIDs";E={$VMUIDs}},
								@{N="VMNames";E={$VMNames}},
								@{N="HostIDs";E={$HostIDs}},
								@{N="HostUIDs";E={$HostUIDs}},
								@{N="HostNames";E={$HostNames}},
								@{N="VMs";E={$DRSGroup.VM}},
								@{N="Hosts";E={$DRSGroup.Host}}
						}
					}
				}

				$ExpandedDRSGroups | Export-Csv -NoTypeInformation -Path $OutputFile5 -Encoding UTF8
			}
		}
	}
	catch
    {
        LogLastException
    }

	try {
		if ($RequiredData -eq "VMMigrationHistory" -or $RequiredData -eq "AllData") {

			LogProgress "Getting VM History Info"

			Get-VMotionRecords | Export-Csv -NoTypeInformation -Path $OutputFile6 -Encoding UTF8
		}
	}
	catch
    {
        LogLastException
    }

	LogProgress "VMware Script Completed"
}

function CalculateInstallDate([string] $Uuid){
    try {
        if ($Uuid) {
            $hexSeconds = [Regex]::Match($Uuid, '^(\w{8,})-').Groups[1].Value
		    $decSeconds = [Convert]::ToInt64($hexSeconds, 16)
		    $dateDate = [TimeZone]::CurrentTimeZone.ToLocalTime(([DateTime]'1/1/1970').AddSeconds($decSeconds))
		    if ($dateDate) {
                return $dateDate.ToString("yyyy-MM-dd HH:mm:ss")
            }
        }
    }
    catch {
    }
    return "";
}

function Get-VMotionRecords {

	# Search for any vmotion-related events
	$eventFilterSpec = New-Object VMware.Vim.EventFilterSpec
	$eventFilterSpec.Category = "info"
	$eventFilterSpec.Time = New-Object VMware.Vim.EventFilterSpecByTime
	$eventFilterSpec.Time.beginTime = (get-date).adddays(-$VMotionHistoryDays)
	$eventFilterSpec.Type = "VmMigratedEvent", "DrsVmMigratedEvent"
		
	$global:DefaultVIServer.ExtensionData.UpdateViewData("Content.EventManager.*")
	$eventMgr = $global:DefaultVIServer.ExtensionData.Content.LinkedView.EventManager

	$allEvents = @()
	$eventNumber = 100
	$ecollectionImpl = Get-View ($eventMgr.CreateCollectorForEvents($eventFilterSpec))
	$ecollection = $ecollectionImpl.ReadNextEvents($eventNumber)
	while($ecollection -ne $null){
		$allEvents += $ecollection
		$ecollection = $ecollectionImpl.ReadNextEvents($eventNumber)
	}
	$ecollectionImpl.DestroyCollector()

	$vMotions = @()
	$vMotions += $allEvents | Select CreatedTime,
			@{N="Type";E={$_.GetType().Name}},
			@{N="TypeId";E={$_.EventTypeId}},
			@{N="ID";E={$_.key}},
			@{N="UserName";E={if($_.UserName){$_.UserName}else{"System"}}},
			@{N="VM";E={$_.VM.Name}},
			@{N="VMId";E={$_.VM.vm.Id}},
			@{N="VMUid";E={$_.VM.vm.Uid}}, 
			@{N="SourceHost";E={$_.SourceHost.Name}},
			@{N="DestinationHost";E={$_.Host.Name}},
			@{N="Description";E={$_.FullFormattedMessage}},  
			@{N="ResourcePool";E={$_.ComputeResource.Name}}

	return $vmotions | Sort-Object -Property CreatedTime		
}

function EnvironmentConfigured {
	if (Get-Command "Connect-VIServer" -errorAction SilentlyContinue){
		return $true}
	else {
		return $false}
}

Get-VMwareVMList
